package com.lyg.system.controller;

import cn.dev33.satoken.util.SaResult;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.lyg.common.Constants;
import com.lyg.common.util.StringUtil;
import com.lyg.framework.log.OpLog;
import com.lyg.system.entity.SysOrg;
import com.lyg.system.entity.SysUser;
import com.lyg.system.service.SysOrgService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 机构表;(sys_org)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2023-1-27
 */
@Api(tags = "机构表对象功能接口")
@RestController
@RequestMapping("/")
public class SysOrgController extends BaseController {

    @Autowired
    private SysOrgService sysOrgService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @OpLog(module = "机构管理", action = "查询详情")
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/sysOrg/{id}")
    public SaResult queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return SaResult.data(sysOrgService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @OpLog(module = "机构管理", action = "查询列表")
    @ApiOperation("分页查询")
    @GetMapping("/sysOrg")
    public SaResult list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                         @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                         @ApiParam("机构名称") @RequestParam(value = "name", required = false) String name) throws Exception {

        IPage<SysOrg> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<SysOrg> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtil.isEmpty(name), "name", name);
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByDesc("sort_no");
        IPage<SysOrg> result = sysOrgService.page(pg, queryWrapper);

        return SaResult.data(result);
    }

    /**
     * 新增数据
     *
     * @param sysOrg 实例对象
     * @return 实例对象
     */
    @OpLog(module = "机构管理", action = "新增数据")
    @ApiOperation("新增数据")
    @PostMapping("/sysOrg")
    public SaResult add(@ApiParam("对象实体") @RequestBody SysOrg sysOrg) throws Exception {
        if (StringUtil.isEmpty(sysOrg.getParentId())) {
            return SaResult.error("未找到父级机构");
        }

        if (StringUtil.isEmpty(sysOrg.getName())) {
            return SaResult.error("机构名称未填写");
        }

        long count = sysOrgService.countBy("name", sysOrg.getName(), true);
        if (count > 0) {
            return SaResult.error("机构名称重复");
        }
        count = sysOrgService.countBy("org_id", sysOrg.getOrgCode(), true);
        if (count > 0) {
            return SaResult.error("机构号重复");
        }


        SysUser user = currentUser();

        sysOrgService.createNew(sysOrg, user);
        return SaResult.data(sysOrg);
    }

    /**
     * 更新数据
     *
     * @param sysOrg 实例对象
     * @return 实例对象
     */
    @OpLog(module = "机构管理", action = "更新数据")
    @ApiOperation("更新数据")
    @PutMapping("/sysOrg/{id}")
    public SaResult edit(@ApiParam("对象实体") @RequestBody SysOrg sysOrg,
                         @ApiParam("对象ID") @PathVariable String id) throws Exception {
//        if (!StpUtil.hasPermission("org."+id)) {
//            return SaResult.error("暂无权限");
//        }

        if (StringUtil.isEmpty(sysOrg.getParentId())) {
            return SaResult.error("未找到父级机构");
        }

        if (StringUtil.isEmpty(sysOrg.getName())) {
            return SaResult.error("机构名称未填写");
        }

        SysOrg exists = sysOrgService.getByButNot("name", sysOrg.getName(), "org_id", id, true);
        if (null != exists) {
            return SaResult.error("名称已存在");
        }

        sysOrg.setOrgId(id);
        sysOrgService.updateData(sysOrg);
        return SaResult.data(sysOrg);
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @OpLog(module = "机构管理", action = "删除数据")
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/sysOrg/{id}")
    public SaResult deleteById(@ApiParam("对象ID") @PathVariable String id) {
        SysOrg sysOrg = sysOrgService.getById(id);
        if (sysOrg == null || Constants.STATUS_DELETE == sysOrg.getStatus()) {
            return SaResult.ok("数据已删除");
        }

        long count = sysUserService.countBy("org_id", id, true);
        if (count > 0) {
            return SaResult.error("请先删除当前机构下所有用户");
        }

        count = sysOrgService.countBy("parent_id", id, true);
        if (count > 0) {
            return SaResult.error("请先删除当前机构下所有子机构");
        }

        sysOrgService.removeLogicById(id);
        return SaResult.ok("success");
    }
}