package com.lyg.system.controller;

import cn.dev33.satoken.util.SaResult;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.lyg.common.Constants;
import com.lyg.common.util.StringUtil;
import com.lyg.system.entity.SysRole;
import com.lyg.system.entity.SysRoleResource;
import com.lyg.framework.log.OpLog;
import com.lyg.system.service.SysRoleResourceService;
import com.lyg.system.service.SysRoleService;
import com.lyg.system.service.SysUserRoleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.List;

/**
 * 系统角色;(sys_role)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2023-2-5
 */
@Api(tags = "系统角色对象功能接口")
@RestController
@RequestMapping("/")
public class SysRoleController extends BaseController {

    @Autowired
    private SysRoleService sysRoleService;

    @Autowired
    private SysUserRoleService sysUserRoleService;

    @Autowired
    private SysRoleResourceService sysRoleResourceService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @OpLog(module = "角色管理", action = "查询详情")
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/sysRole/{id}")
    public SaResult queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return SaResult.data(sysRoleService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @OpLog(module = "角色管理", action = "查询列表")
    @ApiOperation("分页查询")
    @GetMapping("/sysRole")
    public SaResult list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                         @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) throws Exception {

        IPage<SysRole> pg = new Page<>(pageNum, pageSize);

        QueryWrapper<SysRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByAsc("name");
        IPage<SysRole> result = sysRoleService.page(pg, queryWrapper);

        return SaResult.data(result);
    }

    /**
     * 新增数据
     *
     * @param sysRole 实例对象
     * @return 实例对象
     */
    @OpLog(module = "角色管理", action = "新增数据")
    @ApiOperation("新增数据")
    @PostMapping("/sysRole")
    public SaResult add(@ApiParam("对象实体") @RequestBody SysRole sysRole) throws Exception {
        sysRole.setRoleId(null);

        if (StringUtil.isEmpty(sysRole.getName())) {
            return SaResult.error("角色名称不能为空");
        }

        sysRole.setCreateUser(currentUser().getUserId());
        sysRole.setStatus(Constants.STATUS_NORMAL);
        sysRole.setCreateDate(LocalDateTime.now());
        sysRoleService.save(sysRole);
        return SaResult.data(sysRoleService.getById(sysRole.getRoleId()));
    }

    /**
     * 更新数据
     *
     * @param sysRole 实例对象
     * @return 实例对象
     */
    @OpLog(module = "角色管理", action = "更新数据")
    @ApiOperation("更新数据")
    @PutMapping("/sysRole/{id}")
    public SaResult edit(@ApiParam("对象实体") @RequestBody SysRole sysRole,
                         @ApiParam("对象ID") @PathVariable String id) throws Exception {

        if (StringUtil.isEmpty(sysRole.getName())) {
            return SaResult.error("角色名称不能为空");
        }

        sysRole.setRoleId(id);
        sysRoleService.updateById(sysRole);
        return SaResult.data(sysRole);
    }

    /**
     * 授权资源
     *
     * @param roleId            角色ID
     * @param roleResourcesList 授权列表
     * @return 实例对象
     */
    @OpLog(module = "角色管理", action = "授权资源")
    @ApiOperation("新增数据, 包含编辑")
    @PostMapping("/role/{roleId}/resources")
    public SaResult authResource(@ApiParam("角色ID") @PathVariable String roleId,
                                 @ApiParam("授权列表") @RequestBody List<SysRoleResource> roleResourcesList) throws Exception {
        if (null == roleResourcesList) {
            return SaResult.error("未找到授权对象");
        }

        sysRoleResourceService.mergeData(roleId, roleResourcesList);
        return SaResult.data(roleResourcesList);
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @OpLog(module = "角色管理", action = "删除数据")
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/sysRole/{id}")
    public SaResult deleteById(@ApiParam("对象ID") @PathVariable String id) {
        long count = sysUserRoleService.countBy("role_id", id, false);
        if (count > 0) {
            return SaResult.error("请先取消所有用户授权");
        }

        sysRoleService.removeById(id);
        return SaResult.ok("success");
    }
}